@@ -41,7 +41,7 @@
 #include "nsAccessibilityAtoms.h"
 #include "nsAccessibilityService.h"
 #include "nsAccessibleTreeWalker.h"
-#include "nsTextUtils.h"
+#include "nsTextAttrs.h"
 
 #include "nsIClipboard.h"
 #include "nsContentCID.h"
@@ -133,7 +133,8 @@ nsAccessibleWrap(aNode, aShell)
 {
 }
 
-NS_IMETHODIMP nsHyperTextAccessible::GetRole(PRUint32 *aRole)
+nsresult
+nsHyperTextAccessible::GetRoleInternal(PRUint32 *aRole)
 {
   nsCOMPtr<nsIContent> content = do_QueryInterface(mDOMNode);
   if (!content) {
@@ -211,7 +212,7 @@ void nsHyperTextAccessible::CacheChildren()
   // Special case for text entry fields, go directly to editor's root for children
   if (mAccChildCount == eChildCountUninitialized) {
     PRUint32 role;
-    GetRole(&role);
+    GetRoleInternal(&role);
     if (role != nsIAccessibleRole::ROLE_ENTRY && role != nsIAccessibleRole::ROLE_PASSWORD_TEXT) {
       nsAccessible::CacheChildren();
       return;
@@ -230,17 +231,17 @@ void nsHyperTextAccessible::CacheChildren()
       return;
     }
     nsAccessibleTreeWalker walker(mWeakShell, editorRootDOMNode, PR_TRUE);
-    nsCOMPtr<nsPIAccessible> privatePrevAccessible;
+    nsRefPtr<nsAccessible> prevAcc;
     PRInt32 childCount = 0;
     walker.GetFirstChild();
     SetFirstChild(walker.mState.accessible);
 
     while (walker.mState.accessible) {
       ++ childCount;
-      privatePrevAccessible = do_QueryInterface(walker.mState.accessible);
-      privatePrevAccessible->SetParent(this);
+      prevAcc = nsAccUtils::QueryAccessible(walker.mState.accessible);
+      prevAcc->SetParent(this);
       walker.GetNextSibling();
-      privatePrevAccessible->SetNextSibling(walker.mState.accessible);
+      prevAcc->SetNextSibling(walker.mState.accessible);
     }
     mAccChildCount = childCount;
   }
@@ -294,13 +295,13 @@ nsIntRect nsHyperTextAccessible::GetBoundsForString(nsIFrame *aFrame, PRUint32 a
     // Add the point where the string starts to the frameScreenRect
     nsPoint frameTextStartPoint;
     rv = frame->GetPointFromOffset(startContentOffset, &frameTextStartPoint);
-    NS_ENSURE_SUCCESS(rv, nsRect());   
+    NS_ENSURE_SUCCESS(rv, nsIntRect());
     frameScreenRect.x += context->AppUnitsToDevPixels(frameTextStartPoint.x);
 
     // Use the point for the end offset to calculate the width
     nsPoint frameTextEndPoint;
     rv = frame->GetPointFromOffset(startContentOffset + frameSubStringLength, &frameTextEndPoint);
-    NS_ENSURE_SUCCESS(rv, nsRect());   
+    NS_ENSURE_SUCCESS(rv, nsIntRect());
     frameScreenRect.width = context->AppUnitsToDevPixels(frameTextEndPoint.x - frameTextStartPoint.x);
 
     screenRect.UnionRect(frameScreenRect, screenRect);
@@ -438,9 +439,9 @@ nsHyperTextAccessible::GetPosAndText(PRInt32& aStartOffset, PRInt32& aEndOffset,
               *aText += '*'; // Show *'s only for password text
           }
           else {
-            nsCOMPtr<nsPIAccessible> pAcc(do_QueryInterface(accessible));
-            pAcc->AppendTextTo(*aText, startOffset,
-                               substringEndOffset - startOffset);
+            nsRefPtr<nsAccessible> acc(nsAccUtils::QueryAccessible(accessible));
+            acc->AppendTextTo(*aText, startOffset,
+                              substringEndOffset - startOffset);
           }
         }
         if (aBoundsRect) {    // Caller wants the bounds of the text
@@ -1174,17 +1175,8 @@ nsHyperTextAccessible::GetTextAttributes(PRBool aIncludeDefAttrs,
   if (!node)
     return NS_OK;
 
-  // Set 'lang' text attribute.
-  rv =  GetLangTextAttributes(aIncludeDefAttrs, node,
-                              aStartOffset, aEndOffset,
-                              aAttributes ? *aAttributes : nsnull);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  // Set CSS based text attributes.
-  rv = GetCSSTextAttributes(aIncludeDefAttrs, node,
-                            aStartOffset, aEndOffset,
-                            aAttributes ? *aAttributes : nsnull);
-  return rv;
+  nsTextAttrsMgr textAttrsMgr(this, mDOMNode, aIncludeDefAttrs, node);
+  return textAttrsMgr.GetAttributes(*aAttributes, aStartOffset, aEndOffset);
 }
 
 // nsIPersistentProperties
@@ -1204,35 +1196,8 @@ nsHyperTextAccessible::GetDefaultTextAttributes(nsIPersistentProperties **aAttri
   if (!mDOMNode)
     return NS_ERROR_FAILURE;
 
-  nsCOMPtr<nsIDOMElement> element = nsCoreUtils::GetDOMElementFor(mDOMNode);
-
-  nsCSSTextAttr textAttr(PR_TRUE, element, nsnull);
-  while (textAttr.Iterate()) {
-    nsCAutoString name;
-    nsAutoString value, oldValue;
-    if (textAttr.Get(name, value))
-      attributes->SetStringProperty(name, value, oldValue);
-  }
-  
-  nsIFrame *sourceFrame = nsCoreUtils::GetFrameFor(element);
-  NS_ENSURE_STATE(sourceFrame);
-
-  // set font size
-  nsAutoString value;
-  nsFontSizeTextAttr fontSizeTextAttr(sourceFrame, nsnull);
-  fontSizeTextAttr.Get(value);
-  nsAccUtils::SetAccAttr(attributes,
-                         nsAccessibilityAtoms::fontSize, value);
-  
-  value.Truncate();
-  
-  // set font background color
-  nsBackgroundTextAttr backgroundTextAttr(sourceFrame, nsnull);
-  backgroundTextAttr.Get(value);
-  nsAccUtils::SetAccAttr(attributes,
-                         nsAccessibilityAtoms::backgroundColor, value);
-
-  return NS_OK;
+  nsTextAttrsMgr textAttrsMgr(this, mDOMNode, PR_TRUE, nsnull);
+  return textAttrsMgr.GetAttributes(*aAttributes);
 }
 
 nsresult
@@ -1356,12 +1321,12 @@ nsHyperTextAccessible::GetOffsetAtPoint(PRInt32 aX, PRInt32 aY,
   if (!frameScreenRect.Contains(coords.x, coords.y)) {
     return NS_OK;   // Not found, will return -1
   }
-  nsPoint pointInHyperText(coords.x - frameScreenRect.x,
+  nsIntPoint pxInHyperText(coords.x - frameScreenRect.x,
                            coords.y - frameScreenRect.y);
   nsPresContext *context = GetPresContext();
   NS_ENSURE_TRUE(context, NS_ERROR_FAILURE);
-  pointInHyperText.x = context->DevPixelsToAppUnits(pointInHyperText.x);
-  pointInHyperText.y = context->DevPixelsToAppUnits(pointInHyperText.y);
+  nsPoint pointInHyperText(context->DevPixelsToAppUnits(pxInHyperText.x),
+                           context->DevPixelsToAppUnits(pxInHyperText.y));
 
   // Go through the frames to check if each one has the point.
   // When one does, add up the character offsets until we have a match
@@ -1642,23 +1607,46 @@ NS_IMETHODIMP nsHyperTextAccessible::SetCaretOffset(PRInt32 aCaretOffset)
 /*
  * Gets the offset position of the caret (cursor).
  */
-NS_IMETHODIMP nsHyperTextAccessible::GetCaretOffset(PRInt32 *aCaretOffset)
+NS_IMETHODIMP
+nsHyperTextAccessible::GetCaretOffset(PRInt32 *aCaretOffset)
 {
-  *aCaretOffset = 0;
+  *aCaretOffset = -1;
 
+  // No caret if the focused node is not inside this DOM node and this DOM node
+  // is not inside of focused node.
+  PRBool isInsideOfFocusedNode =
+    nsCoreUtils::IsAncestorOf(gLastFocusedNode, mDOMNode);
+
+  if (!isInsideOfFocusedNode && mDOMNode != gLastFocusedNode &&
+      !nsCoreUtils::IsAncestorOf(mDOMNode, gLastFocusedNode))
+    return NS_OK;
+
+  // Turn the focus node and offset of the selection into caret hypretext
+  // offset.
   nsCOMPtr<nsISelection> domSel;
   nsresult rv = GetSelections(nsISelectionController::SELECTION_NORMAL,
                               nsnull, getter_AddRefs(domSel));
   NS_ENSURE_SUCCESS(rv, rv);
 
-  nsCOMPtr<nsIDOMNode> caretNode;
-  rv = domSel->GetFocusNode(getter_AddRefs(caretNode));
+  nsCOMPtr<nsIDOMNode> focusNode;
+  rv = domSel->GetFocusNode(getter_AddRefs(focusNode));
   NS_ENSURE_SUCCESS(rv, rv);
 
-  PRInt32 caretOffset;
-  domSel->GetFocusOffset(&caretOffset);
+  PRInt32 focusOffset;
+  rv = domSel->GetFocusOffset(&focusOffset);
+  NS_ENSURE_SUCCESS(rv, rv);
 
-  return DOMPointToHypertextOffset(caretNode, caretOffset, aCaretOffset);
+  // No caret if this DOM node is inside of focused node but the selection's
+  // focus point is not inside of this DOM node.
+  if (isInsideOfFocusedNode) {
+    nsCOMPtr<nsIDOMNode> resultNode =
+      nsCoreUtils::GetDOMNodeFromDOMPoint(focusNode, focusOffset);
+    if (resultNode != mDOMNode &&
+        !nsCoreUtils::IsAncestorOf(mDOMNode, resultNode))
+      return NS_OK;
+  }
+
+  return DOMPointToHypertextOffset(focusNode, focusOffset, aCaretOffset);
 }
 
 PRInt32 nsHyperTextAccessible::GetCaretLineNumber()
@@ -2031,8 +2019,7 @@ nsHyperTextAccessible::ScrollSubstringToPoint(PRInt32 aStartIndex,
   PRBool initialScrolled = PR_FALSE;
   nsIFrame *parentFrame = frame;
   while ((parentFrame = parentFrame->GetParent())) {
-    nsIScrollableFrame *scrollableFrame = nsnull;
-    CallQueryInterface(parentFrame, &scrollableFrame);
+    nsIScrollableFrame *scrollableFrame = do_QueryFrame(parentFrame);
     if (scrollableFrame) {
       if (!initialScrolled) {
         // Scroll substring to the given point. Turn the point into percents
@@ -2084,7 +2071,7 @@ nsresult nsHyperTextAccessible::ContentToRenderedOffset(nsIFrame *aFrame, PRInt3
 
   gfxSkipChars skipChars;
   gfxSkipCharsIterator iter;
-  // Only get info up to original ofset, we know that will be larger than skipped offset
+  // Only get info up to original offset, we know that will be larger than skipped offset
   nsresult rv = aFrame->GetRenderedText(nsnull, &skipChars, &iter, 0, aContentOffset);
   NS_ENSURE_SUCCESS(rv, rv);
 
@@ -2298,275 +2285,3 @@ nsHyperTextAccessible::GetSpellTextAttribute(nsIDOMNode *aNode,
 
   return NS_OK;
 }
-
-// nsHyperTextAccessible
-nsresult
-nsHyperTextAccessible::GetLangTextAttributes(PRBool aIncludeDefAttrs,
-                                             nsIDOMNode *aSourceNode,
-                                             PRInt32 *aStartHTOffset,
-                                             PRInt32 *aEndHTOffset,
-                                             nsIPersistentProperties *aAttributes)
-{
-  nsCOMPtr<nsIDOMElement> sourceElm(nsCoreUtils::GetDOMElementFor(aSourceNode));
-
-  nsCOMPtr<nsIContent> content(do_QueryInterface(sourceElm));
-  nsCOMPtr<nsIContent> rootContent(do_QueryInterface(mDOMNode));
-
-  nsAutoString lang;
-  nsCoreUtils::GetLanguageFor(content, rootContent, lang);
-
-  nsAutoString rootLang;
-  nsresult rv = GetLanguage(rootLang);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  if (aAttributes) {
-    // Expose 'language' text attribute if the DOM 'lang' attribute is
-    // presented and it's different from the 'lang' attribute on the root
-    // element or we should include default values of text attribute.
-    const nsAString& resultLang = lang.IsEmpty() ? rootLang : lang;
-    if (!resultLang.IsEmpty() && (aIncludeDefAttrs || lang != rootLang))
-      nsAccUtils::SetAccAttr(aAttributes, nsAccessibilityAtoms::language,
-                             resultLang);
-  }
-
-  nsLangTextAttr textAttr(lang, rootContent);
-  return GetRangeForTextAttr(aSourceNode, &textAttr,
-                             aStartHTOffset, aEndHTOffset);
-}
-
-// nsHyperTextAccessible
-nsresult
-nsHyperTextAccessible::GetCSSTextAttributes(PRBool aIncludeDefAttrs,
-                                            nsIDOMNode *aSourceNode,
-                                            PRInt32 *aStartHTOffset,
-                                            PRInt32 *aEndHTOffset,
-                                            nsIPersistentProperties *aAttributes)
-{
-  nsCOMPtr<nsIDOMElement> sourceElm(nsCoreUtils::GetDOMElementFor(aSourceNode));
-  nsCOMPtr<nsIDOMElement> rootElm(nsCoreUtils::GetDOMElementFor(mDOMNode));
-
-  nsCSSTextAttr textAttr(aIncludeDefAttrs, sourceElm, rootElm);
-  while (textAttr.Iterate()) {
-    nsCAutoString name;
-    nsAutoString value, oldValue;
-    if (aAttributes && textAttr.Get(name, value))
-      aAttributes->SetStringProperty(name, value, oldValue);
-
-    nsresult rv = GetRangeForTextAttr(aSourceNode, &textAttr,
-                                      aStartHTOffset, aEndHTOffset);
-    NS_ENSURE_SUCCESS(rv, rv);
-  }
-
-  nsIFrame *sourceFrame = nsCoreUtils::GetFrameFor(sourceElm);
-  if (sourceFrame) {
-    nsIFrame *rootFrame = nsnull;
-
-    if (!aIncludeDefAttrs)
-      rootFrame = nsCoreUtils::GetFrameFor(rootElm);
-
-    nsFontSizeTextAttr fontSizeTextAttr(sourceFrame, rootFrame);
-    nsAutoString value;
-    if (fontSizeTextAttr.Get(value)) {
-      nsAccUtils::SetAccAttr(aAttributes,
-                             nsAccessibilityAtoms::fontSize, value);
-    }
-
-    nsBackgroundTextAttr backgroundTextAttr(sourceFrame, rootFrame);
-    value.Truncate();
-    if (backgroundTextAttr.Get(value)) {
-      nsAccUtils::SetAccAttr(aAttributes,
-                             nsAccessibilityAtoms::backgroundColor, value);
-    }
-
-    nsresult rv = GetRangeForTextAttr(aSourceNode, &backgroundTextAttr,
-                                      aStartHTOffset, aEndHTOffset);
-    return rv;
-  }
-
-  return NS_OK;
-}
-
-// nsHyperTextAccessible
-nsresult
-nsHyperTextAccessible::GetRangeForTextAttr(nsIDOMNode *aNode,
-                                           nsTextAttr *aComparer,
-                                           PRInt32 *aStartHTOffset,
-                                           PRInt32 *aEndHTOffset)
-{
-  nsCOMPtr<nsIDOMElement> rootElm(nsCoreUtils::GetDOMElementFor(mDOMNode));
-  NS_ENSURE_STATE(rootElm);
-
-  nsCOMPtr<nsIDOMNode> tmpNode(aNode);
-  nsCOMPtr<nsIDOMNode> currNode(aNode);
-
-  // Navigate backwards and forwards from current node to the root node to
-  // calculate range bounds for the text attribute. Navigation sequence is the
-  // following:
-  // 1. Navigate through the siblings.
-  // 2. If the traversed sibling has children then navigate from its leaf child
-  //    to it through whole tree of the traversed sibling.
-  // 3. Get the parent and cycle algorithm until the root node.
-
-  // Navigate backwards (find the start offset).
-  while (currNode && currNode != rootElm) {
-    nsCOMPtr<nsIDOMElement> currElm(nsCoreUtils::GetDOMElementFor(currNode));
-    NS_ENSURE_STATE(currElm);
-
-    if (currNode != aNode && !aComparer->Equal(currElm)) {
-      PRInt32 startHTOffset = 0;
-      nsCOMPtr<nsIAccessible> startAcc;
-      nsresult rv = DOMPointToHypertextOffset(tmpNode, -1, &startHTOffset,
-                                              getter_AddRefs(startAcc));
-      NS_ENSURE_SUCCESS(rv, rv);
-
-      if (!startAcc)
-        startHTOffset = 0;
-
-      if (startHTOffset > *aStartHTOffset)
-        *aStartHTOffset = startHTOffset;
-
-      break;
-    }
-
-    currNode->GetPreviousSibling(getter_AddRefs(tmpNode));
-    if (tmpNode) {
-      // Navigate through the subtree of traversed children to calculate
-      // left bound of the range.
-      FindStartOffsetInSubtree(tmpNode, currNode, aComparer, aStartHTOffset);
-    }
-
-    currNode->GetParentNode(getter_AddRefs(tmpNode));
-    currNode.swap(tmpNode);
-  }
-
-  // Navigate forwards (find the end offset).
-  PRBool moveIntoSubtree = PR_TRUE;
-  currNode = aNode;
-  while (currNode && currNode != rootElm) {
-    nsCOMPtr<nsIDOMElement> currElm(nsCoreUtils::GetDOMElementFor(currNode));
-    NS_ENSURE_STATE(currElm);
-
-    // Stop new end offset searching if the given text attribute changes its
-    // value.
-    if (!aComparer->Equal(currElm)) {
-      PRInt32 endHTOffset = 0;
-      nsresult rv = DOMPointToHypertextOffset(currNode, -1, &endHTOffset);
-      NS_ENSURE_SUCCESS(rv, rv);
-
-      if (endHTOffset < *aEndHTOffset)
-        *aEndHTOffset = endHTOffset;
-
-      break;
-    }
-
-    if (moveIntoSubtree) {
-      // Navigate through subtree of traversed node. We use 'moveIntoSubtree'
-      // flag to avoid traversing the same subtree twice.
-      currNode->GetFirstChild(getter_AddRefs(tmpNode));
-      if (tmpNode)
-        FindEndOffsetInSubtree(tmpNode, aComparer, aEndHTOffset);
-    }
-
-    currNode->GetNextSibling(getter_AddRefs(tmpNode));
-    moveIntoSubtree = PR_TRUE;
-    if (!tmpNode) {
-      currNode->GetParentNode(getter_AddRefs(tmpNode));
-      moveIntoSubtree = PR_FALSE;
-    }
-
-    currNode.swap(tmpNode);
-  }
-
-  return NS_OK;
-}
-
-
-PRBool
-nsHyperTextAccessible::FindEndOffsetInSubtree(nsIDOMNode *aCurrNode,
-                                              nsTextAttr *aComparer,
-                                              PRInt32 *aHTOffset)
-{
-  if (!aCurrNode)
-    return PR_FALSE;
-
-  nsCOMPtr<nsIDOMElement> currElm(nsCoreUtils::GetDOMElementFor(aCurrNode));
-  NS_ENSURE_STATE(currElm);
-
-  // If the given text attribute (pointed by nsTextAttr object) changes its
-  // value on the traversed element then fit the end of range.
-  if (!aComparer->Equal(currElm)) {
-    PRInt32 endHTOffset = 0;
-    nsresult rv = DOMPointToHypertextOffset(aCurrNode, -1, &endHTOffset);
-    NS_ENSURE_SUCCESS(rv, rv);
-
-    if (endHTOffset < *aHTOffset)
-      *aHTOffset = endHTOffset;
-
-    return PR_TRUE;
-  }
-
-  // Deeply traverse into the tree to fit the end of range.
-  nsCOMPtr<nsIDOMNode> nextNode;
-  aCurrNode->GetFirstChild(getter_AddRefs(nextNode));
-  if (nextNode) {
-    PRBool res = FindEndOffsetInSubtree(nextNode, aComparer, aHTOffset);
-    if (res)
-      return res;
-  }
-
-  aCurrNode->GetNextSibling(getter_AddRefs(nextNode));
-  if (nextNode) {
-    if (FindEndOffsetInSubtree(nextNode, aComparer, aHTOffset))
-      return PR_TRUE;
-  }
-
-  return PR_FALSE;
-}
-
-PRBool
-nsHyperTextAccessible::FindStartOffsetInSubtree(nsIDOMNode *aCurrNode,
-                                                nsIDOMNode *aPrevNode,
-                                                nsTextAttr *aComparer,
-                                                PRInt32 *aHTOffset)
-{
-  if (!aCurrNode)
-    return PR_FALSE;
-
-  // Find the closest element back to the traversed element.
-  nsCOMPtr<nsIDOMNode> nextNode;
-  aCurrNode->GetLastChild(getter_AddRefs(nextNode));
-  if (nextNode) {
-    if (FindStartOffsetInSubtree(nextNode, aPrevNode, aComparer, aHTOffset))
-      return PR_TRUE;
-  }
-
-  nsCOMPtr<nsIDOMElement> currElm(nsCoreUtils::GetDOMElementFor(aCurrNode));
-  NS_ENSURE_STATE(currElm);
-
-  // If the given text attribute (pointed by nsTextAttr object) changes its
-  // value on the traversed element then fit the start of range.
-  if (!aComparer->Equal(currElm)) {
-    PRInt32 startHTOffset = 0;
-    nsCOMPtr<nsIAccessible> startAcc;
-    nsresult rv = DOMPointToHypertextOffset(aPrevNode, -1, &startHTOffset,
-                                            getter_AddRefs(startAcc));
-    NS_ENSURE_SUCCESS(rv, rv);
-
-    if (!startAcc)
-      startHTOffset = 0;
-
-    if (startHTOffset > *aHTOffset)
-      *aHTOffset = startHTOffset;
-
-    return PR_TRUE;
-  }
-
-  // Moving backwards to find the start of range.
-  aCurrNode->GetPreviousSibling(getter_AddRefs(nextNode));
-  if (nextNode) {
-    if (FindStartOffsetInSubtree(nextNode, aCurrNode, aComparer, aHTOffset))
-      return PR_TRUE;
-  }
-
-  return PR_FALSE;
-}
