@@ -1,4 +1,5 @@
-/* -*- Mode: C++; c-basic-offset: 4; indent-tabs-mode: t; tab-width: 4 -*- */
+/* -*- Mode: C++; c-basic-offset: 4; indent-tabs-mode: nil; tab-width: 4 -*- */
+/* vi: set ts=4 sw=4 expandtab: (add to ~/.vimrc: set modeline modelines=5) */
 /* ***** BEGIN LICENSE BLOCK *****
  * Version: MPL 1.1/GPL 2.0/LGPL 2.1
  *
@@ -64,36 +65,47 @@ namespace nanojit
 
     static const int kLinkageAreaSize = 68;
     static const int kcalleeAreaSize = 80; // The max size.
-    static const int NJ_PAGE_SIZE_SPARC = 8192; // Use sparc page size here.
 
 #define BIT_ROUND_UP(v,q)      ( (((uintptr_t)v)+(q)-1) & ~((q)-1) )
+#define TODO(x) do{ verbose_only(outputf(#x);) NanoAssertMsgf(false, "%s", #x); } while(0)
 
     void Assembler::nInit(AvmCore* core)
     {
         has_cmov = true;
     }
 
+    void Assembler::nBeginAssembly() {
+    }
+
     NIns* Assembler::genPrologue()
     {
         /**
          * Prologue
          */
-        underrunProtect(12);
-        uint32_t stackNeeded = STACK_GRANULARITY * _activation.highwatermark;
+        underrunProtect(16);
+        uint32_t stackNeeded = STACK_GRANULARITY * _activation.tos;
         uint32_t frameSize = stackNeeded + kcalleeAreaSize + kLinkageAreaSize;
         frameSize = BIT_ROUND_UP(frameSize, 8);
 
-        verbose_only( verbose_outputf("        %p:",_nIns); )
-            verbose_only( verbose_output("        patch entry:"); )
-            NIns *patchEntry = _nIns;
         if (frameSize <= 4096)
-            SAVEI(SP, (-frameSize), SP);
+            SUBI(FP, frameSize, SP);
         else {
-            SAVE(SP, G1, SP);
-            ORI(G1, -frameSize & 0x3FF, G1);
-            SETHI(-frameSize, G1);
+            SUB(FP, G1, SP);
+            ORI(G1, frameSize & 0x3FF, G1);
+            SETHI(frameSize, G1);
         }
 
+        verbose_only(
+        if (_logc->lcbits & LC_Assembly) {
+            outputf("        %p:",_nIns);
+            output("        patch entry:");
+        })
+        NIns *patchEntry = _nIns;
+
+        // The frame size in SAVE is faked. We will still re-caculate SP later.
+        // We can use 0 here but it is not good for debuggers.
+        SAVEI(SP, -148, SP);
+
         // align the entry point
         asm_align_code();
 
@@ -118,7 +130,9 @@ namespace nanojit
             }
         else
             {
-                // target doesn't exit yet.  emit jump to epilog, and set up to patch later.
+                // Target doesn't exit yet. Emit jump to epilog, and set up to patch later.
+                if (!_epilogue)
+                    _epilogue = genEpilogue();
                 lr = guard->record();
                 JMP_long((intptr_t)_epilogue);
                 lr->jmp = _nIns;
@@ -136,7 +150,7 @@ namespace nanojit
         ORI(O0, 0, I0);
         return  _nIns;
     }
-    
+
     void Assembler::asm_call(LInsp ins)
     {
         const CallInfo* call = ins->callInfo();
@@ -144,29 +158,23 @@ namespace nanojit
         underrunProtect(8);
         NOP();
 
-        ArgSize sizes[10];
+        ArgSize sizes[MAXARGS];
         uint32_t argc = call->get_sizes(sizes);
 
-        if (ins->isop(LIR_call) || ins->isop(LIR_fcall)) {
-            verbose_only(if (_verbose)
-                         outputf("        %p:", _nIns);
-                         )
-                CALL(call);
+        NanoAssert(ins->isop(LIR_pcall) || ins->isop(LIR_fcall));
+        verbose_only(if (_logc->lcbits & LC_Assembly)
+                     outputf("        %p:", _nIns);
+                     )
+        bool indirect = call->isIndirect();
+        if (!indirect) {
+            CALL(call);
         }
         else {
             argc--;
             Register r = findSpecificRegFor(ins->arg(argc), I0);
-            NanoAssert(ins->isop(LIR_calli) || ins->isop(LIR_fcalli));
             JMPL(G0, I0, 15);
         }
 
-        bool imt = call->isInterface();
-
-        if(imt) {
-            argc--;
-            findSpecificRegFor(ins->arg(argc), O3);
-        }
-
         uint32_t GPRIndex = O0;
         uint32_t offset = kLinkageAreaSize; // start of parameters stack postion.
 
@@ -204,24 +212,7 @@ namespace nanojit
             }
     }
 
-    void Assembler::nMarkExecute(Page* page, int flags)
-    {
-        static const int kProtFlags[4] = {
-            PROT_READ,                        // 0
-            PROT_READ|PROT_WRITE,            // PAGE_WRITE
-            PROT_READ|PROT_EXEC,            // PAGE_EXEC
-            PROT_READ|PROT_WRITE|PROT_EXEC    // PAGE_EXEC|PAGE_WRITE
-        };
-        int prot = kProtFlags[flags & (PAGE_WRITE|PAGE_EXEC)];
-        intptr_t addr = (intptr_t)page;
-        addr &= ~((uintptr_t)NJ_PAGE_SIZE_SPARC - 1);
-        if (mprotect((char *)addr, NJ_PAGE_SIZE_SPARC, prot) == -1) {
-            // todo: we can't abort or assert here, we have to fail gracefully.
-            NanoAssertMsg(false, "FATAL ERROR: mprotect(PROT_EXEC) failed\n");
-        }
-    }
-            
-    Register Assembler::nRegisterAllocFromSet(int set)
+    Register Assembler::nRegisterAllocFromSet(RegisterMask set)
     {
         // need to implement faster way
         int i=0;
@@ -234,21 +225,16 @@ namespace nanojit
     void Assembler::nRegisterResetAll(RegAlloc& a)
     {
         a.clear();
-        a.used = 0;
         a.free = GpRegs | FpRegs;
         debug_only( a.managed = a.free; )
-            }
+    }
 
-    NIns *Assembler::nPatchBranch(NIns* branch, NIns* location)
+    void Assembler::nPatchBranch(NIns* branch, NIns* location)
     {
-        NIns *was;
-
-        was = (NIns*)(((*(uint32_t*)&branch[0] & 0x3FFFFF) << 10) | (*(uint32_t*)&branch[1] & 0x3FF ));
         *(uint32_t*)&branch[0] &= 0xFFC00000;
         *(uint32_t*)&branch[0] |= ((intptr_t)location >> 10) & 0x3FFFFF;
         *(uint32_t*)&branch[1] &= 0xFFFFFC00;
         *(uint32_t*)&branch[1] |= (intptr_t)location & 0x3FF;
-        return was;
     }
 
     RegisterMask Assembler::hint(LIns* i, RegisterMask allow)
@@ -264,15 +250,13 @@ namespace nanojit
         LIns* lo = ins->oprnd1();
         LIns* hi = ins->oprnd2();
 
-        Reservation *resv = getresv(ins);
-        Register rr = resv->reg;
-
-        if (rr != UnknownReg && (rmask(rr) & FpRegs))
-            evict(rr);
+        Register rr = ins->getReg();
+        if (isKnownReg(rr) && (rmask(rr) & FpRegs))
+            evict(rr, ins);
 
         if (hi->isconst()) {
             STW32(L2, d+4, FP);
-            SET32(hi->constval(), L2);
+            SET32(hi->imm32(), L2);
         } else {
             Register rh = findRegFor(hi, GpRegs);
             STW32(rh, d+4, FP);
@@ -280,7 +264,7 @@ namespace nanojit
 
         if (lo->isconst()) {
             STW32(L2, d, FP);
-            SET32(lo->constval(), L2);
+            SET32(lo->imm32(), L2);
         } else {
             // okay if r gets recycled.
             Register rl = findRegFor(lo, GpRegs);
@@ -291,21 +275,22 @@ namespace nanojit
     }
 
 
-    void Assembler::asm_restore(LInsp i, Reservation *resv, Register r)
+    void Assembler::asm_restore(LInsp i, Reservation *unused, Register r)
     {
         underrunProtect(24);
         if (i->isop(LIR_alloc)) {
             ADD(FP, L2, r);
-            SET32(disp(resv), L2);
-            verbose_only(if (_verbose) {
+            int32_t d = disp(i);
+            SET32(d, L2);
+            verbose_only(if (_logc->lcbits & LC_RegAlloc) {
                 outputf("        remat %s size %d", _thisfrag->lirbuf->names->formatRef(i), i->size());
             })
-                }
+        }
         else if (i->isconst()) {
-            if (!resv->arIndex) {
-                reserveFree(i);
+            if (!i->getArIndex()) {
+                i->markAsClear();
             }
-            int v = i->constval();
+            int v = i->imm32();
             SET32(v, r);
         } else {
             int d = findMemFor(i);
@@ -314,10 +299,10 @@ namespace nanojit
             } else {
                 LDSW32(FP, d, r);
             }
-            verbose_only(if (_verbose) {
+            verbose_only(if (_logc->lcbits & LC_RegAlloc) {
                 outputf("        restore %s", _thisfrag->lirbuf->names->formatRef(i));
             })
-                }
+        }
     }
 
     void Assembler::asm_store32(LIns *value, int dr, LIns *base)
@@ -326,14 +311,13 @@ namespace nanojit
         if (value->isconst())
             {
                 Register rb = getBaseReg(base, dr, GpRegs);
-                int c = value->constval();
+                int c = value->imm32();
                 STW32(L2, dr, rb);
                 SET32(c, L2);
             }
         else
             {
                 // make sure what is in a register
-                Reservation *rA, *rB;
                 Register ra, rb;
                 if (base->isop(LIR_alloc)) {
                     rb = FP;
@@ -341,13 +325,11 @@ namespace nanojit
                     ra = findRegFor(value, GpRegs);
                 } else if (base->isconst()) {
                     // absolute address
-                    dr += base->constval();
+                    dr += base->imm32();
                     ra = findRegFor(value, GpRegs);
                     rb = G0;
                 } else {
-                    findRegFor2(GpRegs, value, rA, base, rB);
-                    ra = rA->reg;
-                    rb = rB->reg;
+                    findRegFor2b(GpRegs, value, ra, base, rb);
                 }
                 STW32(ra, dr, rb);
             }
@@ -370,11 +352,10 @@ namespace nanojit
     {
         underrunProtect(72);
         LIns* base = ins->oprnd1();
-        int db = ins->oprnd2()->constval();
-        Reservation *resv = getresv(ins);
-        Register rr = resv->reg;
+        int db = ins->disp();
+        Register rr = ins->getReg();
 
-        int dr = disp(resv);
+        int dr = disp(ins);
         Register rb;
         if (base->isop(LIR_alloc)) {
             rb = FP;
@@ -382,7 +363,7 @@ namespace nanojit
         } else {
             rb = findRegFor(base, GpRegs);
         }
-        resv->reg = UnknownReg;
+        ins->setReg(UnknownReg);
 
         // don't use an fpu reg to simply load & store the value.
         if (dr)
@@ -406,11 +387,10 @@ namespace nanojit
                 // if a constant 64-bit value just store it now rather than
                 // generating a pointless store/load/store sequence
                 Register rb = findRegFor(base, GpRegs);
-                const int32_t* p = (const int32_t*) (value-2);
                 STW32(L2, dr+4, rb);
-                SET32(p[0], L2);
+                SET32(value->imm64_0(), L2);
                 STW32(L2, dr, rb);
-                SET32(p[1], L2);
+                SET32(value->imm64_1(), L2);
                 return;
             }
 
@@ -447,14 +427,9 @@ namespace nanojit
 
         // if value already in a reg, use that, otherwise
         // try to get it into XMM regs before FPU regs.
-        Reservation* rA = getresv(value);
-        Register rv;
-        int pop = !rA || rA->reg==UnknownReg;
-        if (pop) {
-            rv = findRegFor(value, FpRegs);
-        } else {
-            rv = rA->reg;
-        }
+        Register rv = ( value->isUnusedOrHasUnknownReg()
+                      ? findRegFor(value, FpRegs)
+                      : value->getReg() );
 
         STDF32(rv, dr, rb);
     }
@@ -475,9 +450,8 @@ namespace nanojit
         LDSW32(rs, ds, t);
     }
 
-    NIns* Assembler::asm_branch(bool branchOnFalse, LInsp cond, NIns* targ, bool isfar)
+    NIns* Assembler::asm_branch(bool branchOnFalse, LInsp cond, NIns* targ)
     {
-              // XXX ignoring isfar
         NIns* at = 0;
         LOpcode condop = cond->opcode();
         NanoAssert(cond->isCond());
@@ -506,8 +480,6 @@ namespace nanojit
                     BNE(0, tt);
                 else if (condop == LIR_ov)
                     BVC(0, tt);
-                else if (condop == LIR_cs)
-                    BCC(0, tt);
                 else if (condop == LIR_lt)
                     BGE(0, tt);
                 else if (condop == LIR_le)
@@ -531,8 +503,6 @@ namespace nanojit
                     BE(0, tt);
                 else if (condop == LIR_ov)
                     BVS(0, tt);
-                else if (condop == LIR_cs)
-                    BCS(0, tt);
                 else if (condop == LIR_lt)
                     BL(0, tt);
                 else if (condop == LIR_le)
@@ -558,14 +528,13 @@ namespace nanojit
     {
         underrunProtect(12);
         LOpcode condop = cond->opcode();
-        
-        // LIR_ov and LIR_cs recycle the flags set by arithmetic ops
-        if ((condop == LIR_ov) || (condop == LIR_cs))
+
+        // LIR_ov recycles the flags set by arithmetic ops
+        if (condop == LIR_ov)
             return;
-        
+
         LInsp lhs = cond->oprnd1();
         LInsp rhs = cond->oprnd2();
-        Reservation *rA, *rB;
 
         NanoAssert((!lhs->isQuad() && !rhs->isQuad()) || (lhs->isQuad() && rhs->isQuad()));
 
@@ -574,7 +543,7 @@ namespace nanojit
         // ready to issue the compare
         if (rhs->isconst())
             {
-                int c = rhs->constval();
+                int c = rhs->imm32();
                 if (c == 0 && cond->isop(LIR_eq)) {
                     Register r = findRegFor(lhs, GpRegs);
                     ANDCC(r, r, G0);
@@ -587,48 +556,30 @@ namespace nanojit
             }
         else
             {
-                findRegFor2(GpRegs, lhs, rA, rhs, rB);
-                Register ra = rA->reg;
-                Register rb = rB->reg;
+                Register ra, rb;
+                findRegFor2b(GpRegs, lhs, ra, rhs, rb);
                 SUBCC(ra, rb, G0);
             }
     }
 
-    void Assembler::asm_loop(LInsp ins, NInsList& loopJumps)
-    {
-        (void)ins;
-        JMP_long_placeholder(); // jump to SOT    
-        verbose_only( if (_verbose && _outputCache) { _outputCache->removeLast(); outputf("         jmp   SOT"); } );
-        
-        loopJumps.add(_nIns);
-
-        assignSavedRegs();
-
-        // restore first parameter, the only one we use
-        LInsp state = _thisfrag->lirbuf->state;
-        findSpecificRegFor(state, argRegs[state->imm8()]); 
-    }    
-
     void Assembler::asm_fcond(LInsp ins)
     {
-        // only want certain regs 
+        // only want certain regs
         Register r = prepResultReg(ins, AllowableFlagRegs);
         asm_setcc(r, ins);
     }
-                
+
     void Assembler::asm_cond(LInsp ins)
     {
         underrunProtect(8);
-        // only want certain regs 
-        LOpcode op = ins->opcode();            
+        // only want certain regs
+        LOpcode op = ins->opcode();
         Register r = prepResultReg(ins, AllowableFlagRegs);
 
         if (op == LIR_eq)
             MOVEI(1, 1, 0, 0, r);
         else if (op == LIR_ov)
             MOVVSI(1, 1, 0, 0, r);
-        else if (op == LIR_cs)
-            MOVCSI(1, 1, 0, 0, r);
         else if (op == LIR_lt)
             MOVLI(1, 1, 0, 0, r);
         else if (op == LIR_le)
@@ -648,11 +599,11 @@ namespace nanojit
         ORI(G0, 0, r);
         asm_cmp(ins);
     }
-    
+
     void Assembler::asm_arith(LInsp ins)
     {
         underrunProtect(28);
-        LOpcode op = ins->opcode();            
+        LOpcode op = ins->opcode();
         LInsp lhs = ins->oprnd1();
         LInsp rhs = ins->oprnd2();
 
@@ -667,28 +618,27 @@ namespace nanojit
                 }
                 allow &= ~rmask(rb);
             }
-        else if ((op == LIR_add||op == LIR_addp) && lhs->isop(LIR_alloc) && rhs->isconst()) {
+        else if ((op == LIR_add||op == LIR_iaddp) && lhs->isop(LIR_alloc) && rhs->isconst()) {
             // add alloc+const, use lea
             Register rr = prepResultReg(ins, allow);
-            int d = findMemFor(lhs) + rhs->constval();
+            int d = findMemFor(lhs) + rhs->imm32();
             ADD(FP, L2, rr);
             SET32(d, L2);
         }
 
         Register rr = prepResultReg(ins, allow);
-        Reservation* rA = getresv(lhs);
-        Register ra;
         // if this is last use of lhs in reg, we can re-use result reg
-        if (rA == 0 || (ra = rA->reg) == UnknownReg)
-            ra = findSpecificRegFor(lhs, rr);
-        // else, rA already has a register assigned.
+        // else, lhs already has a register assigned.
+        Register ra = ( lhs->isUnusedOrHasUnknownReg()
+                      ? findSpecificRegFor(lhs, rr)
+                      : lhs->getReg() );
 
         if (forceReg)
             {
                 if (lhs == rhs)
                     rb = ra;
 
-                if (op == LIR_add || op == LIR_addp)
+                if (op == LIR_add || op == LIR_iaddp)
                     ADDCC(rr, rb, rr);
                 else if (op == LIR_sub)
                     SUBCC(rr, rb, rr);
@@ -711,11 +661,11 @@ namespace nanojit
             }
         else
             {
-                int c = rhs->constval();
-                if (op == LIR_add || op == LIR_addp) {
-                    ADDCC(rr, L2, rr); 
+                int c = rhs->imm32();
+                if (op == LIR_add || op == LIR_iaddp) {
+                    ADDCC(rr, L2, rr);
                 } else if (op == LIR_sub) {
-                    SUBCC(rr, L2, rr); 
+                    SUBCC(rr, L2, rr);
                 } else if (op == LIR_and)
                     AND(rr, L2, rr);
                 else if (op == LIR_or)
@@ -733,41 +683,39 @@ namespace nanojit
                 SET32(c, L2);
             }
 
-        if ( rr != ra ) 
+        if ( rr != ra )
             ORI(ra, 0, rr);
     }
-    
+
     void Assembler::asm_neg_not(LInsp ins)
     {
         underrunProtect(8);
-        LOpcode op = ins->opcode();            
+        LOpcode op = ins->opcode();
         Register rr = prepResultReg(ins, GpRegs);
 
         LIns* lhs = ins->oprnd1();
-        Reservation *rA = getresv(lhs);
         // if this is last use of lhs in reg, we can re-use result reg
-        Register ra;
-        if (rA == 0 || (ra=rA->reg) == UnknownReg)
-            ra = findSpecificRegFor(lhs, rr);
-        // else, rA already has a register assigned.
+        // else, lhs already has a register assigned.
+        Register ra = ( lhs->isUnusedOrHasUnknownReg()
+                      ? findSpecificRegFor(lhs, rr)
+                      : lhs->getReg() );
 
         if (op == LIR_not)
-            ORN(G0, rr, rr); 
+            ORN(G0, rr, rr);
         else
-            SUB(G0, rr, rr); 
+            SUB(G0, rr, rr);
 
-        if ( rr != ra ) 
+        if ( rr != ra )
             ORI(ra, 0, rr);
     }
-                
+
     void Assembler::asm_ld(LInsp ins)
     {
         underrunProtect(12);
-        LOpcode op = ins->opcode();            
+        LOpcode op = ins->opcode();
         LIns* base = ins->oprnd1();
-        LIns* disp = ins->oprnd2();
+        int d = ins->disp();
         Register rr = prepResultReg(ins, GpRegs);
-        int d = disp->constval();
         Register ra = getBaseReg(base, d, GpRegs);
         if (op == LIR_ldcb) {
             LDUB32(ra, d, rr);
@@ -781,18 +729,14 @@ namespace nanojit
     void Assembler::asm_cmov(LInsp ins)
     {
         underrunProtect(4);
-        LOpcode op = ins->opcode();            
+        LOpcode op = ins->opcode();
         LIns* condval = ins->oprnd1();
+        LIns* iftrue  = ins->oprnd2();
+        LIns* iffalse = ins->oprnd3();
+
         NanoAssert(condval->isCmp());
-
-        LIns* values = ins->oprnd2();
-
-        NanoAssert(values->opcode() == LIR_2);
-        LIns* iftrue = values->oprnd1();
-        LIns* iffalse = values->oprnd2();
-
         NanoAssert(op == LIR_qcmov || (!iftrue->isQuad() && !iffalse->isQuad()));
-        
+
         const Register rr = prepResultReg(ins, GpRegs);
 
         // this code assumes that neither LD nor MR nor MRcc set any of the condition flags.
@@ -803,7 +747,6 @@ namespace nanojit
                 // note that these are all opposites...
             case LIR_eq:  MOVNE (iffalsereg, 1, 0, 0, rr); break;
             case LIR_ov:  MOVVC (iffalsereg, 1, 0, 0, rr); break;
-            case LIR_cs:  MOVCC (iffalsereg, 1, 0, 0, rr); break;
             case LIR_lt:  MOVGE (iffalsereg, 1, 0, 0, rr); break;
             case LIR_le:  MOVG  (iffalsereg, 1, 0, 0, rr); break;
             case LIR_gt:  MOVLE (iffalsereg, 1, 0, 0, rr); break;
@@ -820,7 +763,7 @@ namespace nanojit
         /*const Register iftruereg =*/ findSpecificRegFor(iftrue, rr);
         asm_cmp(condval);
     }
-                
+
     void Assembler::asm_qhi(LInsp ins)
     {
         underrunProtect(12);
@@ -832,8 +775,8 @@ namespace nanojit
 
     void Assembler::asm_param(LInsp ins)
     {
-        uint32_t a = ins->imm8();
-        uint32_t kind = ins->imm8b();
+        uint32_t a = ins->paramArg();
+        uint32_t kind = ins->paramKind();
         //        prepResultReg(ins, rmask(argRegs[a]));
         if (kind == 0) {
             prepResultReg(ins, rmask(argRegs[a]));
@@ -842,17 +785,6 @@ namespace nanojit
         }
     }
 
-    void Assembler::asm_short(LInsp ins)
-    {
-        underrunProtect(8);
-        Register rr = prepResultReg(ins, GpRegs);
-        int32_t val = ins->imm16();
-        if (val == 0)
-            XOR(rr, rr, rr);
-        else
-            SET32(val, rr);
-    }
-
     void Assembler::asm_int(LInsp ins)
     {
         underrunProtect(8);
@@ -867,34 +799,30 @@ namespace nanojit
     void Assembler::asm_quad(LInsp ins)
     {
         underrunProtect(64);
-        Reservation *rR = getresv(ins);
-        Register rr = rR->reg;
+        Register rr = ins->getReg();
         if (rr != UnknownReg)
             {
                 // @todo -- add special-cases for 0 and 1
                 _allocator.retire(rr);
-                rR->reg = UnknownReg;
+                ins->setReg(UnknownReg);
                 NanoAssert((rmask(rr) & FpRegs) != 0);
                 findMemFor(ins);
-                int d = disp(rR);
+                int d = disp(ins);
                 LDDF32(FP, d, rr);
             }
 
         // @todo, if we used xor, ldsd, fldz, etc above, we don't need mem here
-        int d = disp(rR);
+        int d = disp(ins);
         freeRsrcOf(ins, false);
         if (d)
             {
-                Register r = registerAlloc(GpRegs);
-                _allocator.addFree(r);
-                const int32_t* p = (const int32_t*) (ins-2);
-                STW32(r, d+4, FP);
-                SET32(p[0], r);
-                STW32(r, d, FP);
-                SET32(p[1], r);
+                STW32(L2, d+4, FP);
+                SET32(ins->imm64_0(), L2);
+                STW32(L2, d, FP);
+                SET32(ins->imm64_1(), L2);
             }
     }
-    
+
     void Assembler::asm_qlo(LInsp ins)
     {
     }
@@ -906,14 +834,11 @@ namespace nanojit
         LIns* lhs = ins->oprnd1();
 
         // lhs into reg, prefer same reg as result
-        Reservation* rA = getresv(lhs);
-        Register ra;
         // if this is last use of lhs in reg, we can re-use result reg
-        if (rA == 0 || rA->reg == UnknownReg)
-            ra = findSpecificRegFor(lhs, rr);
-        else
-            ra = findRegFor(lhs, FpRegs);
-        // else, rA already has a different reg assigned
+        // else, lhs already has a different reg assigned
+        Register ra = ( lhs->isUnusedOrHasUnknownReg()
+                      ? findSpecificRegFor(lhs, rr)
+                      : findRegFor(lhs, FpRegs) );
 
         FNEGD(ra, rr);
     }
@@ -952,12 +877,11 @@ namespace nanojit
         LDDF32(FP, d, rr);
     }
 
-    Register Assembler::asm_prep_fcall(Reservation *rR, LInsp ins)
+    Register Assembler::asm_prep_fcall(Reservation *unused, LInsp ins)
     {
-        if (rR) {
-            Register rr;
-            if ((rr=rR->reg) != UnknownReg && (rmask(rr) & FpRegs))
-                evict(rr);
+        Register rr;
+        if (ins->isUsed() && (rr = ins->getReg(), isKnownReg(rr)) && (rmask(rr) & FpRegs)) {
+            evict(rr, ins);
         }
         return prepResultReg(ins, rmask(F0));
     }
@@ -1012,13 +936,13 @@ namespace nanojit
                 if (condop == LIR_feq)
                     FBNE(0, tt);
                 else if (condop == LIR_fle)
-                    FBG(0, tt);
+                    FBUG(0, tt);
                 else if (condop == LIR_flt)
-                    FBGE(0, tt);
+                    FBUGE(0, tt);
                 else if (condop == LIR_fge)
-                    FBL(0, tt);
+                    FBUL(0, tt);
                 else //if (condop == LIR_fgt)
-                    FBLE(0, tt);
+                    FBULE(0, tt);
             }
         else // op == LIR_xt
             {
@@ -1067,52 +991,49 @@ namespace nanojit
 
         FCMPD(rLhs, rRhs);
     }
-    
-/** no longer called by patch/unpatch
-    NIns* Assembler::asm_adjustBranch(NIns* at, NIns* target)
+
+    verbose_only(
+    void Assembler::asm_inc_m32(uint32_t* pCtr)
     {
-        NIns* was;
-        was = (NIns*)(((*(uint32_t*)&at[0] & 0x3FFFFF) << 10) | (*(uint32_t*)&at[1] & 0x3FF ));
-        *(uint32_t*)&at[0] &= 0xFFC00000;
-        *(uint32_t*)&at[0] |= ((intptr_t)target >> 10) & 0x3FFFFF;
-        *(uint32_t*)&at[1] &= 0xFFFFFC00;
-        *(uint32_t*)&at[1] |= (intptr_t)target & 0x3FF;
-        return was;
+        // TODO(asm_inc_m32);
     }
- */
-    
+    )
+
     void Assembler::nativePageReset()
     {
     }
 
     Register Assembler::asm_binop_rhs_reg(LInsp ins)
     {
-        return UnknownReg;    
+        return UnknownReg;
     }
 
     void Assembler::nativePageSetup()
     {
-        if (!_nIns)         _nIns       = pageAlloc();
-        if (!_nExitIns)  _nExitIns = pageAlloc(true);
+        if (!_nIns)
+            codeAlloc(codeStart, codeEnd, _nIns verbose_only(, codeBytes));
+        if (!_nExitIns)
+            codeAlloc(exitStart, exitEnd, _nExitIns verbose_only(, exitBytes));
     }
 
     void
-    Assembler::underrunProtect(int bytes)
+    Assembler::underrunProtect(int n)
     {
-        intptr_t u = bytes + sizeof(PageHeader)/sizeof(NIns) + 16;
-        if (!samepage((intptr_t)_nIns-u,_nIns)) {
-            NIns* target = _nIns;
-            _nIns = pageAlloc(_inExit);
-            JMP_long_nocheck((intptr_t)target);
+        NIns *eip = _nIns;
+        if (eip - n < (_inExit ? exitStart : codeStart)) {
+            // We are done with the current page.  Tell Valgrind that new code
+            // has been generated.
+            if (_inExit)
+                codeAlloc(exitStart, exitEnd, _nIns verbose_only(, exitBytes));
+            else
+                codeAlloc(codeStart, codeEnd, _nIns verbose_only(, codeBytes));
+            JMP_long_nocheck((intptr_t)eip);
         }
     }
 
-/*
     void Assembler::asm_ret(LInsp ins)
     {
-        if (_nIns != _epilogue) {
-            JMP(_epilogue);
-        }
+        genEpilogue();
         assignSavedRegs();
         LIns *val = ins->oprnd1();
         if (ins->isop(LIR_ret)) {
@@ -1121,7 +1042,11 @@ namespace nanojit
             findSpecificRegFor(val, F0);
         }
     }
-*/
+
+    void Assembler::asm_promote(LIns *) {
+        // i2q or u2q
+        TODO(asm_promote);
+    }
 
 #endif /* FEATURE_NANOJIT */
 }
