@@ -20,7 +20,7 @@
  * Portions created by the Initial Developer are Copyright (C) 2006
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): Igor Bukanov
+ * Contributor(s): Jesse Ruderman
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -36,23 +36,23 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-gTestfile = 'regress-354145-07.js';
+gTestfile = 'regress-327691-01.js';
 
-var BUGNUMBER = 354145;
-var summary = 'Immutable XML';
+var summary = "Do not crash in js_IsXMLName";
+var BUGNUMBER = 327691;
 var actual = 'No Crash';
 var expect = 'No Crash';
 
 printBugNumber(BUGNUMBER);
 START(summary);
 
+var A=<y/>;
+var B=A.h;
+var D=B.h;
+B.h=D.h;
+B[0]=B.h;
 
-var a = <a><b/></a>
-try {
-  a.insertChildAfter(a.b[0], {toString: function() { throw 1; }});
-} catch (e) { }
-
-for each (var i in a) { }
+B.h=D.h; // Crashes at js_IsXMLName.
 
 TEST(1, expect, actual);
 
