@@ -1,4 +1,4 @@
-/* -*- Mode: java; tab-width:8; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
+/* -*- Mode: java; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /* ***** BEGIN LICENSE BLOCK *****
  * Version: MPL 1.1/GPL 2.0/LGPL 2.1
  *
@@ -15,13 +15,11 @@
  * The Original Code is JavaScript Engine testing utilities.
  *
  * The Initial Developer of the Original Code is
- * David P. Caldwell.
- * Portions created by the Initial Developer are Copyright (C) 2007
+ * Mozilla Foundation.
+ * Portions created by the Initial Developer are Copyright (C) 2005
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s):
- *   Martin Honnen
- *   David P. Caldwell <inonit@inonit.com>
+ * Contributor(s): Seno.Aiko@gmail.com
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -37,16 +35,28 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-gTestfile = '13.4.3.10.js';
+gTestfile = '13.4.4.3-02.js';
 
-START("13.4.3.10 - XML Constructor [[HasInstance]]");
+var summary = "13.4.4.3 - XML.appendChild should copy child";
+var BUGNUMBER = 312692;
+var actual = '';
+var expect = 'error';
 
-printBugNumber(288027);
+printBugNumber(BUGNUMBER);
+START(summary);
 
-var xmlListObject1 = new XMLList('<god>Kibo</god>');
-var xmlListObject2 = new XMLList('<god>Kibo</god><devil>Xibo</devil>');
+try
+{
+    var node = <node/>;
+    node.appendChild(node);
 
-TEST(1, true, xmlListObject1 instanceof XML);
-TEST(2, true, xmlListObject2 instanceof XML);
+    var result = String(node);
+    actual = 'no error';
+}
+catch(e)
+{
+    actual = 'error';
+}
+TEST(1, expect, actual);
 
 END();
