@@ -16,10 +16,10 @@
  *
  * The Initial Developer of the Original Code is
  * Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2005
+ * Portions created by the Initial Developer are Copyright (C) 2006
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): Jesse Ruderman
+ * Contributor(s):
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -35,51 +35,30 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-var gTestfile = 'regress-347306-01.js';
+var gTestfile = 'regress-360969-04.js';
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 347306;
-var summary = 'decompilation should not be O(N**2)';
-var actual = '';
-var expect = '';
+var BUGNUMBER = 360969;
+var summary = '2^17: global const';
+var actual = 'No Crash';
+var expect = 'No Crash';
+
+var global = this;
 
 printBugNumber(BUGNUMBER);
 printStatus (summary);
+ 
+var start = new Date();
+var p;
+var i;
+var limit = 2 << 16;
 
-var data = {X:[], Y:[]};
-for (var size = 1000; size <= 10000; size += 1000)
-{ 
-  data.X.push(size);
-  data.Y.push(testSource(size));
-  gc();
-}
-
-var order = BigO(data);
-
-var msg = '';
-for (var p = 0; p < data.X.length; p++)
+for (var i = 0; i < limit; i++)
 {
-  msg += '(' + data.X[p] + ', ' + data.Y[p] + '); ';
+  eval('const pv' + i + ';');
 }
-printStatus(msg);
-printStatus('Order: ' + order);
-reportCompare(true, order < 2, 'BigO ' + order + ' < 2');
 
-function testSource(n)
-{
-  var funtext = "";
- 
-  for (var i=0; i<n; ++i)
-    funtext += "alert(" + i + "); ";
+reportCompare(expect, actual, summary);
 
-  var fun = new Function(funtext);
+var stop = new Date();
 
-  var start = new Date();
- 
-  var s = fun + "";
- 
-  var end = new Date();
-
-  print("Size: " + n + ", Time: " + (end - start) + " ms");
-
-  return end - start;
-}
+print('Elapsed time: ' + Math.floor((stop - start)/1000) + ' seconds');
