@@ -19,7 +19,7 @@
  * Portions created by the Initial Developer are Copyright (C) 2008
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): Igor Bukanov
+ * Contributor(s): Jesse Ruderman
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -35,32 +35,31 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-var gTestfile = 'regress-462734-04.js';
+var gTestfile = 'regress-470300-01.js';
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 462734;
-var summary = 'Do not assert: pobj_ == obj2';
+var BUGNUMBER = 470300;
+var summary = 'TM: Do not assert: StackBase(fp) + blockDepth == regs.sp';
 var actual = '';
 var expect = '';
 
-printBugNumber(BUGNUMBER);
-printStatus (summary);
 
-var save__proto__ = __proto__;
+//-----------------------------------------------------------------------------
+test();
+//-----------------------------------------------------------------------------
 
-try
+function test()
 {
-  Function.prototype.prototype = function() { return 42; }
-  __proto__ = Function();
-  prototype = prototype;
+  enterFunc ('test');
+  printBugNumber(BUGNUMBER);
+  printStatus (summary);
+ 
+  jit(true);
+
+  for (let a = 0; a < 3; ++a) { let b = '' + []; }
+
+  jit(false);
+
+  reportCompare(expect, actual, summary);
+
+  exitFunc ('test');
 }
-catch(ex)
-{
-  print(ex + '');
-}
-
-expect = 'object';
-actual = typeof prototype;
-
-__proto__ = save__proto__;
-
-reportCompare(expect, actual, summary);
