@@ -19,7 +19,7 @@
  * Portions created by the Initial Developer are Copyright (C) 2008
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): Jesse Ruderman
+ * Contributor(s): Gary Kwong
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -35,7 +35,7 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-var gTestfile = 'regress-452498-076.js';
+var gTestfile = 'regress-452498-102.js';
 //-----------------------------------------------------------------------------
 var BUGNUMBER = 452498;
 var summary = 'TM: upvar2 regression tests';
@@ -52,13 +52,69 @@ function test()
   enterFunc ('test');
   printBugNumber(BUGNUMBER);
   printStatus (summary);
+ 
+// ------- Comment #102 From Gary Kwong [:nth10sd]
 
-// ------- Comment #76 From Jesse Ruderman
+// =====
 
-  for (let d = 0; d < 4; ++d) { d; }
-// 1: ReferenceError: d is not defined
+  (function(){function x(){} function x()y})();
+
+// Assertion failure: JOF_OPTYPE(op) == JOF_ATOM, at ../jsemit.cpp:1710
+
+// =====
+  function f() {
+    "" + (function(){
+        for( ; [function(){}] ; x = 0)
+          with({x: ""})
+            const x = []
+            });
+  }
+  f();
+
+// Assertion failure: ss->top - saveTop <= 1U, at ../jsopcode.cpp:2156
+
+// =====
+
+  try
+  {
+    function f() {
+      var x;
+      eval("const x = [];");
+    }
+    f();
+  }
+  catch(ex)
+  {
+  }
+// Assertion failure: regs.sp == StackBase(fp), at ../jsinterp.cpp:2984
+
+// =====
+  try
+  {
+    do {x} while([[] for (x in []) ]);
+  }
+  catch(ex)
+  {
+  }
+// Assertion failure: !(pnu->pn_dflags & PND_BOUND), at ../jsemit.cpp:1818
+// =====
+
+  try
+  {
+    {x} ((x=[] for (x in []))); x;
+  }
+  catch(ex)
+  {
+  }
+// Assertion failure: cg->staticLevel >= level, at ../jsemit.cpp:2014
+// Crash [@ BindNameToSlot] in opt without -j
+
+// =====
 
   reportCompare(expect, actual, summary);
 
   exitFunc ('test');
 }
+
+
+
